// -*- mode: C++ -*-

// AUTOGENERATED BY glean_parser.  DO NOT EDIT.

/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef mozilla_GleanJSMetricsLookup_h
#define mozilla_GleanJSMetricsLookup_h

#include "mozilla/PerfectHash.h"
#include "mozilla/Maybe.h"
#include "mozilla/glean/bindings/MetricTypes.h"
#include "mozilla/glean/fog_ffi_generated.h"

#define GLEAN_INDEX_BITS (32)
#define GLEAN_TYPE_BITS (5)
#define GLEAN_ID_BITS (27)
#define GLEAN_TYPE_ID(id) ((id) >> GLEAN_ID_BITS)
#define GLEAN_METRIC_ID(id) ((id) & ((1ULL << GLEAN_ID_BITS) - 1))
#define GLEAN_OFFSET(entry) (entry & ((1ULL << GLEAN_INDEX_BITS) - 1))

namespace mozilla::glean {

// The category lookup table's entry type
using category_entry_t = uint32_t;
// The metric lookup table's entry type
// This is a bitpacked type with 32 bits available to index into
// the string table, 5 bits available to signify the metric type,
// and the remaining 27 bits devoted to 2 "signal"
// bits to signify important characteristics (metric's a labeled metric's
// submetric, metric's been registered at runtime) and 25 bits
// for built-in metric ids.
// Gives room for 33554432 of each combination of
// characteristics (which hopefully will prove to be enough).
using metric_entry_t = uint64_t;

static_assert(GLEAN_INDEX_BITS + GLEAN_TYPE_BITS + GLEAN_ID_BITS == sizeof(metric_entry_t) * 8, "Index, Type, and ID bits need to fit into a metric_entry_t");
static_assert(GLEAN_TYPE_BITS + GLEAN_ID_BITS <= sizeof(uint32_t) * 8, "Metric Types and IDs need to fit into at most 32 bits");
static_assert(36 < UINT32_MAX, "Too many metric categories generated.");
static_assert(179 < 33554432, "Too many metrics generated. Need room for 2 signal bits.");
static_assert(19 < 32, "Too many different metric types.");

static already_AddRefed<nsISupports> NewMetricFromId(uint32_t id) {
  uint32_t typeId = GLEAN_TYPE_ID(id);
  uint32_t metricId = GLEAN_METRIC_ID(id);

  switch (typeId) {
    case 1: /* labeled_counter */
    {
      return MakeAndAddRef<GleanLabeled>(metricId, 1);
    }
    case 2: /* denominator */
    {
      return MakeAndAddRef<GleanDenominator>(metricId);
    }
    case 3: /* numerator */
    {
      return MakeAndAddRef<GleanNumerator>(metricId);
    }
    case 4: /* event */
    {
      return MakeAndAddRef<GleanEvent>(metricId);
    }
    case 5: /* timing_distribution */
    {
      return MakeAndAddRef<GleanTimingDistribution>(metricId);
    }
    case 6: /* labeled_boolean */
    {
      return MakeAndAddRef<GleanLabeled>(metricId, 6);
    }
    case 7: /* timespan */
    {
      return MakeAndAddRef<GleanTimespan>(metricId);
    }
    case 8: /* boolean */
    {
      return MakeAndAddRef<GleanBoolean>(metricId);
    }
    case 9: /* counter */
    {
      return MakeAndAddRef<GleanCounter>(metricId);
    }
    case 10: /* memory_distribution */
    {
      return MakeAndAddRef<GleanMemoryDistribution>(metricId);
    }
    case 11: /* custom_distribution */
    {
      return MakeAndAddRef<GleanCustomDistribution>(metricId);
    }
    case 12: /* string */
    {
      return MakeAndAddRef<GleanString>(metricId);
    }
    case 13: /* url */
    {
      return MakeAndAddRef<GleanUrl>(metricId);
    }
    case 14: /* uuid */
    {
      return MakeAndAddRef<GleanUuid>(metricId);
    }
    case 15: /* quantity */
    {
      return MakeAndAddRef<GleanQuantity>(metricId);
    }
    case 16: /* datetime */
    {
      return MakeAndAddRef<GleanDatetime>(metricId);
    }
    case 17: /* string_list */
    {
      return MakeAndAddRef<GleanStringList>(metricId);
    }
    case 18: /* labeled_string */
    {
      return MakeAndAddRef<GleanLabeled>(metricId, 18);
    }
    case 19: /* rate */
    {
      return MakeAndAddRef<GleanRate>(metricId);
    }
    default:
    MOZ_ASSERT_UNREACHABLE("Invalid type ID reached when trying to instantiate a new metric");
    return nullptr;
  }
}

/**
 * Create a submetric instance for a labeled metric of the provided type and id for the given label.
 * Assigns or retrieves an id for the submetric from the SDK.
 *
 * @param aParentTypeId - The type of the parent labeled metric identified as a number generated during codegen.
 *                        Only used to identify which X of LabeledX you are so that X can be created here.
 * @param aParentMetricId - The metric id for the parent labeled metric.
 * @param aLabel - The label for the submetric. Might not adhere to the SDK label format.
 * @param aSubmetricId - an outparam which is assigned the submetric's SDK-generated submetric id.
 *                       Used only by GIFFT.
 */
static already_AddRefed<nsISupports> NewSubMetricFromIds(uint32_t aParentTypeId, uint32_t aParentMetricId, const nsACString& aLabel, uint32_t* aSubmetricId) {
  switch (aParentTypeId) {
    case 1: { /* labeled_counter */
      auto id = impl::fog_labeled_counter_get(aParentMetricId, &aLabel);
      *aSubmetricId = id;
      return MakeAndAddRef<GleanCounter>(id);
    }
    case 6: { /* labeled_boolean */
      auto id = impl::fog_labeled_boolean_get(aParentMetricId, &aLabel);
      *aSubmetricId = id;
      return MakeAndAddRef<GleanBoolean>(id);
    }
    case 18: { /* labeled_string */
      auto id = impl::fog_labeled_string_get(aParentMetricId, &aLabel);
      *aSubmetricId = id;
      return MakeAndAddRef<GleanString>(id);
    }
    default: {
      MOZ_ASSERT_UNREACHABLE("Invalid type ID for submetric.");
      return nullptr;
    }
  }
}

static Maybe<uint32_t> category_result_check(const nsACString& aKey, category_entry_t entry);
static Maybe<uint32_t> metric_result_check(const nsACString& aKey, metric_entry_t entry);

#if defined(_MSC_VER) && !defined(__clang__)
const char gCategoryStringTable[] = {
#else
constexpr char gCategoryStringTable[] = {
#endif
  /*     0 - "gmp" */ 'g', 'm', 'p', '\0',
  /*     4 - "rtcrtpsender" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', '\0',
  /*    17 - "rtcrtpsenderSetparameters" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '\0',
  /*    43 - "perf" */ 'p', 'e', 'r', 'f', '\0',
  /*    48 - "paint" */ 'p', 'a', 'i', 'n', 't', '\0',
  /*    54 - "wr" */ 'w', 'r', '\0',
  /*    57 - "netwerk" */ 'n', 'e', 't', 'w', 'e', 'r', 'k', '\0',
  /*    65 - "cookieBanners" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '\0',
  /*    79 - "cookieBannersClick" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', 'C', 'l', 'i', 'c', 'k', '\0',
  /*    98 - "fog" */ 'f', 'o', 'g', '\0',
  /*   102 - "fogIpc" */ 'f', 'o', 'g', 'I', 'p', 'c', '\0',
  /*   109 - "pdfjs" */ 'p', 'd', 'f', 'j', 's', '\0',
  /*   115 - "power" */ 'p', 'o', 'w', 'e', 'r', '\0',
  /*   121 - "powerWakeupsPerThread" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '\0',
  /*   143 - "powerCpuMsPerThread" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '\0',
  /*   163 - "powerBattery" */ 'p', 'o', 'w', 'e', 'r', 'B', 'a', 't', 't', 'e', 'r', 'y', '\0',
  /*   176 - "browserLaunchedToHandle" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'L', 'a', 'u', 'n', 'c', 'h', 'e', 'd', 'T', 'o', 'H', 'a', 'n', 'd', 'l', 'e', '\0',
  /*   200 - "newtab" */ 'n', 'e', 'w', 't', 'a', 'b', '\0',
  /*   207 - "newtabSearch" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', '\0',
  /*   220 - "topsites" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '\0',
  /*   229 - "pocket" */ 'p', 'o', 'c', 'k', 'e', 't', '\0',
  /*   236 - "newtabSearchAd" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', 'A', 'd', '\0',
  /*   251 - "urlbar" */ 'u', 'r', 'l', 'b', 'a', 'r', '\0',
  /*   258 - "browserEngagement" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'E', 'n', 'g', 'a', 'g', 'e', 'm', 'e', 'n', 't', '\0',
  /*   276 - "pingCentre" */ 'p', 'i', 'n', 'g', 'C', 'e', 'n', 't', 'r', 'e', '\0',
  /*   287 - "extensions" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', '\0',
  /*   298 - "nimbusEvents" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '\0',
  /*   311 - "searchEngineDefault" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '\0',
  /*   331 - "searchEnginePrivate" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '\0',
  /*   351 - "dap" */ 'd', 'a', 'p', '\0',
  /*   355 - "legacyTelemetry" */ 'l', 'e', 'g', 'a', 'c', 'y', 'T', 'e', 'l', 'e', 'm', 'e', 't', 'r', 'y', '\0',
  /*   371 - "backgroundUpdate" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '\0',
  /*   388 - "update" */ 'u', 'p', 'd', 'a', 't', 'e', '\0',
  /*   395 - "testOnly" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '\0',
  /*   404 - "testOnlyIpc" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '\0',
  /*   416 - "testOnlyJog" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'J', 'o', 'g', '\0',
};


static_assert(sizeof(gCategoryStringTable) < UINT32_MAX, "Category string table is too large.");

const category_entry_t sCategoryByNameLookupEntries[] = {
  229ul,
  4ul,
  388ul,
  404ul,
  17ul,
  416ul,
  65ul,
  0ul,
  287ul,
  258ul,
  48ul,
  98ul,
  371ul,
  236ul,
  121ul,
  251ul,
  311ul,
  220ul,
  143ul,
  102ul,
  298ul,
  276ul,
  351ul,
  207ul,
  163ul,
  355ul,
  115ul,
  57ul,
  54ul,
  176ul,
  79ul,
  43ul,
  200ul,
  395ul,
  331ul,
  109ul
};



static Maybe<uint32_t>
CategoryByNameLookup(const nsACString& aKey)
{
  static const uint8_t BASES[] = {
       0,   0,   0,   0,   0,   0,   1,   1,   1,   0,   3,   2,   1,   1,   1,   1,
       2,   2,   1,   1,   2,   1,   4,   0,   0,   0,   2,   0,   0,   0,   9,   0,
       1,   0,   0,   0,   0,   0,   8,   2,   0,   0,   0,   0,  13,   0,   0,   2,
       0,   1,   0,   2,   3,   0,   0,   3,   0,   0,  46,   0,  24,   0,   0,   1,
  };
  

  const char* bytes = aKey.BeginReading();
  size_t length = aKey.Length();
  auto& entry = mozilla::perfecthash::Lookup(bytes, length, BASES,
                                             sCategoryByNameLookupEntries);
  return category_result_check(aKey, entry);
}


#if defined(_MSC_VER) && !defined(__clang__)
const char gMetricStringTable[] = {
#else
constexpr char gMetricStringTable[] = {
#endif
  /*     0 - "gmp.updateXmlFetchResult" */ 'g', 'm', 'p', '.', 'u', 'p', 'd', 'a', 't', 'e', 'X', 'm', 'l', 'F', 'e', 't', 'c', 'h', 'R', 'e', 's', 'u', 'l', 't', '\0',
  /*    25 - "rtcrtpsender.count" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', '.', 'c', 'o', 'u', 'n', 't', '\0',
  /*    44 - "rtcrtpsender.countSetparametersCompat" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', '.', 'c', 'o', 'u', 'n', 't', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', 'C', 'o', 'm', 'p', 'a', 't', '\0',
  /*    82 - "rtcrtpsender.usedSendencodings" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', '.', 'u', 's', 'e', 'd', 'S', 'e', 'n', 'd', 'e', 'n', 'c', 'o', 'd', 'i', 'n', 'g', 's', '\0',
  /*   113 - "rtcrtpsenderSetparameters.warnNoGetparameters" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'w', 'a', 'r', 'n', 'N', 'o', 'G', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '\0',
  /*   159 - "rtcrtpsenderSetparameters.warnLengthChanged" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'w', 'a', 'r', 'n', 'L', 'e', 'n', 'g', 't', 'h', 'C', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*   203 - "rtcrtpsenderSetparameters.warnRidChanged" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'w', 'a', 'r', 'n', 'R', 'i', 'd', 'C', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*   244 - "rtcrtpsenderSetparameters.warnNoTransactionid" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'w', 'a', 'r', 'n', 'N', 'o', 'T', 'r', 'a', 'n', 's', 'a', 'c', 't', 'i', 'o', 'n', 'i', 'd', '\0',
  /*   290 - "rtcrtpsenderSetparameters.warnStaleTransactionid" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'w', 'a', 'r', 'n', 'S', 't', 'a', 'l', 'e', 'T', 'r', 'a', 'n', 's', 'a', 'c', 't', 'i', 'o', 'n', 'i', 'd', '\0',
  /*   339 - "rtcrtpsenderSetparameters.failLengthChanged" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'L', 'e', 'n', 'g', 't', 'h', 'C', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*   383 - "rtcrtpsenderSetparameters.failRidChanged" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'R', 'i', 'd', 'C', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*   424 - "rtcrtpsenderSetparameters.failNoGetparameters" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'N', 'o', 'G', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '\0',
  /*   470 - "rtcrtpsenderSetparameters.failNoTransactionid" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'N', 'o', 'T', 'r', 'a', 'n', 's', 'a', 'c', 't', 'i', 'o', 'n', 'i', 'd', '\0',
  /*   516 - "rtcrtpsenderSetparameters.failStaleTransactionid" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'S', 't', 'a', 'l', 'e', 'T', 'r', 'a', 'n', 's', 'a', 'c', 't', 'i', 'o', 'n', 'i', 'd', '\0',
  /*   565 - "rtcrtpsenderSetparameters.failNoEncodings" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'N', 'o', 'E', 'n', 'c', 'o', 'd', 'i', 'n', 'g', 's', '\0',
  /*   607 - "rtcrtpsenderSetparameters.failOther" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'O', 't', 'h', 'e', 'r', '\0',
  /*   643 - "perf.pageLoad" */ 'p', 'e', 'r', 'f', '.', 'p', 'a', 'g', 'e', 'L', 'o', 'a', 'd', '\0',
  /*   657 - "paint.buildDisplaylistTime" */ 'p', 'a', 'i', 'n', 't', '.', 'b', 'u', 'i', 'l', 'd', 'D', 'i', 's', 'p', 'l', 'a', 'y', 'l', 'i', 's', 't', 'T', 'i', 'm', 'e', '\0',
  /*   684 - "wr.gpuWaitTime" */ 'w', 'r', '.', 'g', 'p', 'u', 'W', 'a', 'i', 't', 'T', 'i', 'm', 'e', '\0',
  /*   699 - "wr.rasterizeGlyphsTime" */ 'w', 'r', '.', 'r', 'a', 's', 't', 'e', 'r', 'i', 'z', 'e', 'G', 'l', 'y', 'p', 'h', 's', 'T', 'i', 'm', 'e', '\0',
  /*   722 - "wr.rasterizeBlobsTime" */ 'w', 'r', '.', 'r', 'a', 's', 't', 'e', 'r', 'i', 'z', 'e', 'B', 'l', 'o', 'b', 's', 'T', 'i', 'm', 'e', '\0',
  /*   744 - "wr.rendererTime" */ 'w', 'r', '.', 'r', 'e', 'n', 'd', 'e', 'r', 'e', 'r', 'T', 'i', 'm', 'e', '\0',
  /*   760 - "wr.rendererTimeNoSc" */ 'w', 'r', '.', 'r', 'e', 'n', 'd', 'e', 'r', 'e', 'r', 'T', 'i', 'm', 'e', 'N', 'o', 'S', 'c', '\0',
  /*   780 - "wr.framebuildTime" */ 'w', 'r', '.', 'f', 'r', 'a', 'm', 'e', 'b', 'u', 'i', 'l', 'd', 'T', 'i', 'm', 'e', '\0',
  /*   798 - "wr.scenebuildTime" */ 'w', 'r', '.', 's', 'c', 'e', 'n', 'e', 'b', 'u', 'i', 'l', 'd', 'T', 'i', 'm', 'e', '\0',
  /*   816 - "wr.sceneswapTime" */ 'w', 'r', '.', 's', 'c', 'e', 'n', 'e', 's', 'w', 'a', 'p', 'T', 'i', 'm', 'e', '\0',
  /*   833 - "wr.textureCacheUpdateTime" */ 'w', 'r', '.', 't', 'e', 'x', 't', 'u', 'r', 'e', 'C', 'a', 'c', 'h', 'e', 'U', 'p', 'd', 'a', 't', 'e', 'T', 'i', 'm', 'e', '\0',
  /*   859 - "wr.timeToFrameBuild" */ 'w', 'r', '.', 't', 'i', 'm', 'e', 'T', 'o', 'F', 'r', 'a', 'm', 'e', 'B', 'u', 'i', 'l', 'd', '\0',
  /*   879 - "wr.timeToRenderStart" */ 'w', 'r', '.', 't', 'i', 'm', 'e', 'T', 'o', 'R', 'e', 'n', 'd', 'e', 'r', 'S', 't', 'a', 'r', 't', '\0',
  /*   900 - "netwerk.earlyHints" */ 'n', 'e', 't', 'w', 'e', 'r', 'k', '.', 'e', 'a', 'r', 'l', 'y', 'H', 'i', 'n', 't', 's', '\0',
  /*   919 - "netwerk.ehLinkType" */ 'n', 'e', 't', 'w', 'e', 'r', 'k', '.', 'e', 'h', 'L', 'i', 'n', 'k', 'T', 'y', 'p', 'e', '\0',
  /*   938 - "cookieBanners.normalWindowServiceMode" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 'n', 'o', 'r', 'm', 'a', 'l', 'W', 'i', 'n', 'd', 'o', 'w', 'S', 'e', 'r', 'v', 'i', 'c', 'e', 'M', 'o', 'd', 'e', '\0',
  /*   976 - "cookieBanners.privateWindowServiceMode" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 'p', 'r', 'i', 'v', 'a', 't', 'e', 'W', 'i', 'n', 'd', 'o', 'w', 'S', 'e', 'r', 'v', 'i', 'c', 'e', 'M', 'o', 'd', 'e', '\0',
  /*  1015 - "cookieBanners.ruleLookupByLoad" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 'r', 'u', 'l', 'e', 'L', 'o', 'o', 'k', 'u', 'p', 'B', 'y', 'L', 'o', 'a', 'd', '\0',
  /*  1046 - "cookieBanners.ruleLookupByDomain" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 'r', 'u', 'l', 'e', 'L', 'o', 'o', 'k', 'u', 'p', 'B', 'y', 'D', 'o', 'm', 'a', 'i', 'n', '\0',
  /*  1079 - "cookieBanners.reload" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 'r', 'e', 'l', 'o', 'a', 'd', '\0',
  /*  1100 - "cookieBannersClick.handleDuration" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', 'C', 'l', 'i', 'c', 'k', '.', 'h', 'a', 'n', 'd', 'l', 'e', 'D', 'u', 'r', 'a', 't', 'i', 'o', 'n', '\0',
  /*  1134 - "cookieBannersClick.result" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', 'C', 'l', 'i', 'c', 'k', '.', 'r', 'e', 's', 'u', 'l', 't', '\0',
  /*  1160 - "fog.initialization" */ 'f', 'o', 'g', '.', 'i', 'n', 'i', 't', 'i', 'a', 'l', 'i', 'z', 'a', 't', 'i', 'o', 'n', '\0',
  /*  1179 - "fog.failedIdleRegistration" */ 'f', 'o', 'g', '.', 'f', 'a', 'i', 'l', 'e', 'd', 'I', 'd', 'l', 'e', 'R', 'e', 'g', 'i', 's', 't', 'r', 'a', 't', 'i', 'o', 'n', '\0',
  /*  1206 - "fogIpc.replayFailures" */ 'f', 'o', 'g', 'I', 'p', 'c', '.', 'r', 'e', 'p', 'l', 'a', 'y', 'F', 'a', 'i', 'l', 'u', 'r', 'e', 's', '\0',
  /*  1228 - "fogIpc.bufferSizes" */ 'f', 'o', 'g', 'I', 'p', 'c', '.', 'b', 'u', 'f', 'f', 'e', 'r', 'S', 'i', 'z', 'e', 's', '\0',
  /*  1247 - "fogIpc.flushDurations" */ 'f', 'o', 'g', 'I', 'p', 'c', '.', 'f', 'l', 'u', 's', 'h', 'D', 'u', 'r', 'a', 't', 'i', 'o', 'n', 's', '\0',
  /*  1269 - "fogIpc.flushFailures" */ 'f', 'o', 'g', 'I', 'p', 'c', '.', 'f', 'l', 'u', 's', 'h', 'F', 'a', 'i', 'l', 'u', 'r', 'e', 's', '\0',
  /*  1290 - "fogIpc.shutdownRegistrationFailures" */ 'f', 'o', 'g', 'I', 'p', 'c', '.', 's', 'h', 'u', 't', 'd', 'o', 'w', 'n', 'R', 'e', 'g', 'i', 's', 't', 'r', 'a', 't', 'i', 'o', 'n', 'F', 'a', 'i', 'l', 'u', 'r', 'e', 's', '\0',
  /*  1326 - "pdfjs.editing" */ 'p', 'd', 'f', 'j', 's', '.', 'e', 'd', 'i', 't', 'i', 'n', 'g', '\0',
  /*  1340 - "pdfjs.buttons" */ 'p', 'd', 'f', 'j', 's', '.', 'b', 'u', 't', 't', 'o', 'n', 's', '\0',
  /*  1354 - "pdfjs.used" */ 'p', 'd', 'f', 'j', 's', '.', 'u', 's', 'e', 'd', '\0',
  /*  1365 - "pdfjs.timeToView" */ 'p', 'd', 'f', 'j', 's', '.', 't', 'i', 'm', 'e', 'T', 'o', 'V', 'i', 'e', 'w', '\0',
  /*  1382 - "power.cpuTimeBogusValues" */ 'p', 'o', 'w', 'e', 'r', '.', 'c', 'p', 'u', 'T', 'i', 'm', 'e', 'B', 'o', 'g', 'u', 's', 'V', 'a', 'l', 'u', 'e', 's', '\0',
  /*  1407 - "power.cpuTimePerProcessTypeMs" */ 'p', 'o', 'w', 'e', 'r', '.', 'c', 'p', 'u', 'T', 'i', 'm', 'e', 'P', 'e', 'r', 'P', 'r', 'o', 'c', 'e', 's', 's', 'T', 'y', 'p', 'e', 'M', 's', '\0',
  /*  1437 - "power.cpuTimePerTrackerTypeMs" */ 'p', 'o', 'w', 'e', 'r', '.', 'c', 'p', 'u', 'T', 'i', 'm', 'e', 'P', 'e', 'r', 'T', 'r', 'a', 'c', 'k', 'e', 'r', 'T', 'y', 'p', 'e', 'M', 's', '\0',
  /*  1467 - "power.gpuTimePerProcessTypeMs" */ 'p', 'o', 'w', 'e', 'r', '.', 'g', 'p', 'u', 'T', 'i', 'm', 'e', 'P', 'e', 'r', 'P', 'r', 'o', 'c', 'e', 's', 's', 'T', 'y', 'p', 'e', 'M', 's', '\0',
  /*  1497 - "power.gpuTimeBogusValues" */ 'p', 'o', 'w', 'e', 'r', '.', 'g', 'p', 'u', 'T', 'i', 'm', 'e', 'B', 'o', 'g', 'u', 's', 'V', 'a', 'l', 'u', 'e', 's', '\0',
  /*  1522 - "power.wakeupsPerProcessType" */ 'p', 'o', 'w', 'e', 'r', '.', 'w', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'P', 'r', 'o', 'c', 'e', 's', 's', 'T', 'y', 'p', 'e', '\0',
  /*  1550 - "power.totalCpuTimeMs" */ 'p', 'o', 'w', 'e', 'r', '.', 't', 'o', 't', 'a', 'l', 'C', 'p', 'u', 'T', 'i', 'm', 'e', 'M', 's', '\0',
  /*  1571 - "power.totalGpuTimeMs" */ 'p', 'o', 'w', 'e', 'r', '.', 't', 'o', 't', 'a', 'l', 'G', 'p', 'u', 'T', 'i', 'm', 'e', 'M', 's', '\0',
  /*  1592 - "power.totalThreadWakeups" */ 'p', 'o', 'w', 'e', 'r', '.', 't', 'o', 't', 'a', 'l', 'T', 'h', 'r', 'e', 'a', 'd', 'W', 'a', 'k', 'e', 'u', 'p', 's', '\0',
  /*  1617 - "powerWakeupsPerThread.parentActive" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'p', 'a', 'r', 'e', 'n', 't', 'A', 'c', 't', 'i', 'v', 'e', '\0',
  /*  1652 - "powerWakeupsPerThread.parentInactive" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'p', 'a', 'r', 'e', 'n', 't', 'I', 'n', 'a', 'c', 't', 'i', 'v', 'e', '\0',
  /*  1689 - "powerWakeupsPerThread.contentForeground" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'c', 'o', 'n', 't', 'e', 'n', 't', 'F', 'o', 'r', 'e', 'g', 'r', 'o', 'u', 'n', 'd', '\0',
  /*  1729 - "powerWakeupsPerThread.contentBackground" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'c', 'o', 'n', 't', 'e', 'n', 't', 'B', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', '\0',
  /*  1769 - "powerWakeupsPerThread.gpuProcess" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'g', 'p', 'u', 'P', 'r', 'o', 'c', 'e', 's', 's', '\0',
  /*  1802 - "powerCpuMsPerThread.parentActive" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'p', 'a', 'r', 'e', 'n', 't', 'A', 'c', 't', 'i', 'v', 'e', '\0',
  /*  1835 - "powerCpuMsPerThread.parentInactive" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'p', 'a', 'r', 'e', 'n', 't', 'I', 'n', 'a', 'c', 't', 'i', 'v', 'e', '\0',
  /*  1870 - "powerCpuMsPerThread.contentForeground" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'c', 'o', 'n', 't', 'e', 'n', 't', 'F', 'o', 'r', 'e', 'g', 'r', 'o', 'u', 'n', 'd', '\0',
  /*  1908 - "powerCpuMsPerThread.contentBackground" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'c', 'o', 'n', 't', 'e', 'n', 't', 'B', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', '\0',
  /*  1946 - "powerCpuMsPerThread.gpuProcess" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'g', 'p', 'u', 'P', 'r', 'o', 'c', 'e', 's', 's', '\0',
  /*  1977 - "powerBattery.percentageWhenUserActive" */ 'p', 'o', 'w', 'e', 'r', 'B', 'a', 't', 't', 'e', 'r', 'y', '.', 'p', 'e', 'r', 'c', 'e', 'n', 't', 'a', 'g', 'e', 'W', 'h', 'e', 'n', 'U', 's', 'e', 'r', 'A', 'c', 't', 'i', 'v', 'e', '\0',
  /*  2015 - "browserLaunchedToHandle.systemNotification" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'L', 'a', 'u', 'n', 'c', 'h', 'e', 'd', 'T', 'o', 'H', 'a', 'n', 'd', 'l', 'e', '.', 's', 'y', 's', 't', 'e', 'm', 'N', 'o', 't', 'i', 'f', 'i', 'c', 'a', 't', 'i', 'o', 'n', '\0',
  /*  2058 - "newtab.locale" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'l', 'o', 'c', 'a', 'l', 'e', '\0',
  /*  2072 - "newtab.newtabCategory" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'n', 'e', 'w', 't', 'a', 'b', 'C', 'a', 't', 'e', 'g', 'o', 'r', 'y', '\0',
  /*  2094 - "newtab.homepageCategory" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'h', 'o', 'm', 'e', 'p', 'a', 'g', 'e', 'C', 'a', 't', 'e', 'g', 'o', 'r', 'y', '\0',
  /*  2118 - "newtab.opened" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'o', 'p', 'e', 'n', 'e', 'd', '\0',
  /*  2132 - "newtab.closed" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'c', 'l', 'o', 's', 'e', 'd', '\0',
  /*  2146 - "newtabSearch.enabled" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', '.', 'e', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  2167 - "newtabSearch.issued" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', '.', 'i', 's', 's', 'u', 'e', 'd', '\0',
  /*  2187 - "topsites.enabled" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '.', 'e', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  2204 - "topsites.sponsoredEnabled" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '.', 's', 'p', 'o', 'n', 's', 'o', 'r', 'e', 'd', 'E', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  2230 - "topsites.impression" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '.', 'i', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', '\0',
  /*  2250 - "topsites.click" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '.', 'c', 'l', 'i', 'c', 'k', '\0',
  /*  2265 - "pocket.isSignedIn" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 'i', 's', 'S', 'i', 'g', 'n', 'e', 'd', 'I', 'n', '\0',
  /*  2283 - "pocket.enabled" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 'e', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  2298 - "pocket.sponsoredStoriesEnabled" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 's', 'p', 'o', 'n', 's', 'o', 'r', 'e', 'd', 'S', 't', 'o', 'r', 'i', 'e', 's', 'E', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  2329 - "pocket.impression" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 'i', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', '\0',
  /*  2347 - "pocket.click" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 'c', 'l', 'i', 'c', 'k', '\0',
  /*  2360 - "pocket.save" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 's', 'a', 'v', 'e', '\0',
  /*  2372 - "pocket.topicClick" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 't', 'o', 'p', 'i', 'c', 'C', 'l', 'i', 'c', 'k', '\0',
  /*  2390 - "newtabSearchAd.impression" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', 'A', 'd', '.', 'i', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', '\0',
  /*  2416 - "newtabSearchAd.click" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', 'A', 'd', '.', 'c', 'l', 'i', 'c', 'k', '\0',
  /*  2437 - "urlbar.abandonment" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'a', 'b', 'a', 'n', 'd', 'o', 'n', 'm', 'e', 'n', 't', '\0',
  /*  2456 - "urlbar.engagement" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'e', 'n', 'g', 'a', 'g', 'e', 'm', 'e', 'n', 't', '\0',
  /*  2474 - "urlbar.impression" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'i', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', '\0',
  /*  2492 - "browserEngagement.activeTicks" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'E', 'n', 'g', 'a', 'g', 'e', 'm', 'e', 'n', 't', '.', 'a', 'c', 't', 'i', 'v', 'e', 'T', 'i', 'c', 'k', 's', '\0',
  /*  2522 - "browserEngagement.uriCount" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'E', 'n', 'g', 'a', 'g', 'e', 'm', 'e', 'n', 't', '.', 'u', 'r', 'i', 'C', 'o', 'u', 'n', 't', '\0',
  /*  2549 - "pingCentre.sendFailures" */ 'p', 'i', 'n', 'g', 'C', 'e', 'n', 't', 'r', 'e', '.', 's', 'e', 'n', 'd', 'F', 'a', 'i', 'l', 'u', 'r', 'e', 's', '\0',
  /*  2573 - "extensions.startupCacheLoadTime" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', '.', 's', 't', 'a', 'r', 't', 'u', 'p', 'C', 'a', 'c', 'h', 'e', 'L', 'o', 'a', 'd', 'T', 'i', 'm', 'e', '\0',
  /*  2605 - "nimbusEvents.enrollment" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'e', 'n', 'r', 'o', 'l', 'l', 'm', 'e', 'n', 't', '\0',
  /*  2629 - "nimbusEvents.enrollFailed" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'e', 'n', 'r', 'o', 'l', 'l', 'F', 'a', 'i', 'l', 'e', 'd', '\0',
  /*  2655 - "nimbusEvents.unenrollment" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'u', 'n', 'e', 'n', 'r', 'o', 'l', 'l', 'm', 'e', 'n', 't', '\0',
  /*  2681 - "nimbusEvents.unenrollFailed" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'u', 'n', 'e', 'n', 'r', 'o', 'l', 'l', 'F', 'a', 'i', 'l', 'e', 'd', '\0',
  /*  2709 - "nimbusEvents.exposure" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'e', 'x', 'p', 'o', 's', 'u', 'r', 'e', '\0',
  /*  2731 - "nimbusEvents.validationFailed" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'v', 'a', 'l', 'i', 'd', 'a', 't', 'i', 'o', 'n', 'F', 'a', 'i', 'l', 'e', 'd', '\0',
  /*  2761 - "searchEngineDefault.engineId" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 'e', 'n', 'g', 'i', 'n', 'e', 'I', 'd', '\0',
  /*  2790 - "searchEngineDefault.displayName" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 'd', 'i', 's', 'p', 'l', 'a', 'y', 'N', 'a', 'm', 'e', '\0',
  /*  2822 - "searchEngineDefault.loadPath" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 'l', 'o', 'a', 'd', 'P', 'a', 't', 'h', '\0',
  /*  2851 - "searchEngineDefault.submissionUrl" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 's', 'u', 'b', 'm', 'i', 's', 's', 'i', 'o', 'n', 'U', 'r', 'l', '\0',
  /*  2885 - "searchEngineDefault.verified" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 'v', 'e', 'r', 'i', 'f', 'i', 'e', 'd', '\0',
  /*  2914 - "searchEngineDefault.changed" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 'c', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*  2942 - "searchEnginePrivate.engineId" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 'e', 'n', 'g', 'i', 'n', 'e', 'I', 'd', '\0',
  /*  2971 - "searchEnginePrivate.displayName" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 'd', 'i', 's', 'p', 'l', 'a', 'y', 'N', 'a', 'm', 'e', '\0',
  /*  3003 - "searchEnginePrivate.loadPath" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 'l', 'o', 'a', 'd', 'P', 'a', 't', 'h', '\0',
  /*  3032 - "searchEnginePrivate.submissionUrl" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 's', 'u', 'b', 'm', 'i', 's', 's', 'i', 'o', 'n', 'U', 'r', 'l', '\0',
  /*  3066 - "searchEnginePrivate.verified" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 'v', 'e', 'r', 'i', 'f', 'i', 'e', 'd', '\0',
  /*  3095 - "searchEnginePrivate.changed" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 'c', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*  3123 - "dap.uploadStatus" */ 'd', 'a', 'p', '.', 'u', 'p', 'l', 'o', 'a', 'd', 'S', 't', 'a', 't', 'u', 's', '\0',
  /*  3140 - "dap.reportGenerationStatus" */ 'd', 'a', 'p', '.', 'r', 'e', 'p', 'o', 'r', 't', 'G', 'e', 'n', 'e', 'r', 'a', 't', 'i', 'o', 'n', 'S', 't', 'a', 't', 'u', 's', '\0',
  /*  3167 - "legacyTelemetry.clientId" */ 'l', 'e', 'g', 'a', 'c', 'y', 'T', 'e', 'l', 'e', 'm', 'e', 't', 'r', 'y', '.', 'c', 'l', 'i', 'e', 'n', 't', 'I', 'd', '\0',
  /*  3192 - "backgroundUpdate.clientId" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'c', 'l', 'i', 'e', 'n', 't', 'I', 'd', '\0',
  /*  3218 - "backgroundUpdate.targetingExists" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'E', 'x', 'i', 's', 't', 's', '\0',
  /*  3251 - "backgroundUpdate.targetingException" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'E', 'x', 'c', 'e', 'p', 't', 'i', 'o', 'n', '\0',
  /*  3287 - "backgroundUpdate.targetingVersion" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'V', 'e', 'r', 's', 'i', 'o', 'n', '\0',
  /*  3321 - "backgroundUpdate.targetingEnvFirefoxVersion" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'E', 'n', 'v', 'F', 'i', 'r', 'e', 'f', 'o', 'x', 'V', 'e', 'r', 's', 'i', 'o', 'n', '\0',
  /*  3365 - "backgroundUpdate.targetingEnvCurrentDate" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'E', 'n', 'v', 'C', 'u', 'r', 'r', 'e', 'n', 't', 'D', 'a', 't', 'e', '\0',
  /*  3406 - "backgroundUpdate.targetingEnvProfileAge" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'E', 'n', 'v', 'P', 'r', 'o', 'f', 'i', 'l', 'e', 'A', 'g', 'e', '\0',
  /*  3446 - "backgroundUpdate.finalState" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'f', 'i', 'n', 'a', 'l', 'S', 't', 'a', 't', 'e', '\0',
  /*  3474 - "backgroundUpdate.states" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 's', 't', 'a', 't', 'e', 's', '\0',
  /*  3498 - "backgroundUpdate.reasons" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'r', 'e', 'a', 's', 'o', 'n', 's', '\0',
  /*  3523 - "backgroundUpdate.exitCodeSuccess" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'e', 'x', 'i', 't', 'C', 'o', 'd', 'e', 'S', 'u', 'c', 'c', 'e', 's', 's', '\0',
  /*  3556 - "backgroundUpdate.exitCodeException" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'e', 'x', 'i', 't', 'C', 'o', 'd', 'e', 'E', 'x', 'c', 'e', 'p', 't', 'i', 'o', 'n', '\0',
  /*  3591 - "update.serviceEnabled" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 's', 'e', 'r', 'v', 'i', 'c', 'e', 'E', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  3613 - "update.autoDownload" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'a', 'u', 't', 'o', 'D', 'o', 'w', 'n', 'l', 'o', 'a', 'd', '\0',
  /*  3633 - "update.backgroundUpdate" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '\0',
  /*  3657 - "update.enabled" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'e', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  3672 - "update.channel" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'c', 'h', 'a', 'n', 'n', 'e', 'l', '\0',
  /*  3687 - "update.canUsuallyApplyUpdates" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'c', 'a', 'n', 'U', 's', 'u', 'a', 'l', 'l', 'y', 'A', 'p', 'p', 'l', 'y', 'U', 'p', 'd', 'a', 't', 'e', 's', '\0',
  /*  3717 - "update.canUsuallyCheckForUpdates" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'c', 'a', 'n', 'U', 's', 'u', 'a', 'l', 'l', 'y', 'C', 'h', 'e', 'c', 'k', 'F', 'o', 'r', 'U', 'p', 'd', 'a', 't', 'e', 's', '\0',
  /*  3750 - "update.canUsuallyStageUpdates" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'c', 'a', 'n', 'U', 's', 'u', 'a', 'l', 'l', 'y', 'S', 't', 'a', 'g', 'e', 'U', 'p', 'd', 'a', 't', 'e', 's', '\0',
  /*  3780 - "update.canUsuallyUseBits" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'c', 'a', 'n', 'U', 's', 'u', 'a', 'l', 'l', 'y', 'U', 's', 'e', 'B', 'i', 't', 's', '\0',
  /*  3805 - "testOnly.badCode" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'b', 'a', 'd', 'C', 'o', 'd', 'e', '\0',
  /*  3822 - "testOnly.canWeTimeIt" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'c', 'a', 'n', 'W', 'e', 'T', 'i', 'm', 'e', 'I', 't', '\0',
  /*  3843 - "testOnly.cheesyString" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'c', 'h', 'e', 'e', 's', 'y', 'S', 't', 'r', 'i', 'n', 'g', '\0',
  /*  3865 - "testOnly.cheesyStringList" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'c', 'h', 'e', 'e', 's', 'y', 'S', 't', 'r', 'i', 'n', 'g', 'L', 'i', 's', 't', '\0',
  /*  3891 - "testOnly.whatADate" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'w', 'h', 'a', 't', 'A', 'D', 'a', 't', 'e', '\0',
  /*  3910 - "testOnly.whatIdIt" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'w', 'h', 'a', 't', 'I', 'd', 'I', 't', '\0',
  /*  3928 - "testOnly.canWeFlagIt" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'c', 'a', 'n', 'W', 'e', 'F', 'l', 'a', 'g', 'I', 't', '\0',
  /*  3949 - "testOnly.doYouRemember" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'd', 'o', 'Y', 'o', 'u', 'R', 'e', 'm', 'e', 'm', 'b', 'e', 'r', '\0',
  /*  3972 - "testOnly.whatTimeIsIt" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'w', 'h', 'a', 't', 'T', 'i', 'm', 'e', 'I', 's', 'I', 't', '\0',
  /*  3994 - "testOnly.mabelsKitchenCounters" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'K', 'i', 't', 'c', 'h', 'e', 'n', 'C', 'o', 'u', 'n', 't', 'e', 'r', 's', '\0',
  /*  4025 - "testOnly.mabelsBathroomCounters" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'B', 'a', 't', 'h', 'r', 'o', 'o', 'm', 'C', 'o', 'u', 'n', 't', 'e', 'r', 's', '\0',
  /*  4057 - "testOnly.mabelsLikeBalloons" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'L', 'i', 'k', 'e', 'B', 'a', 'l', 'l', 'o', 'o', 'n', 's', '\0',
  /*  4085 - "testOnly.mabelsBalloonStrings" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'B', 'a', 'l', 'l', 'o', 'o', 'n', 'S', 't', 'r', 'i', 'n', 'g', 's', '\0',
  /*  4115 - "testOnly.mabelsLabelMaker" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'L', 'a', 'b', 'e', 'l', 'M', 'a', 'k', 'e', 'r', '\0',
  /*  4141 - "testOnly.mirrorTime" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'i', 'r', 'r', 'o', 'r', 'T', 'i', 'm', 'e', '\0',
  /*  4161 - "testOnly.mirrorTimeNanos" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'i', 'r', 'r', 'o', 'r', 'T', 'i', 'm', 'e', 'N', 'a', 'n', 'o', 's', '\0',
  /*  4186 - "testOnly.mirrorsForLabeledBools" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'i', 'r', 'r', 'o', 'r', 's', 'F', 'o', 'r', 'L', 'a', 'b', 'e', 'l', 'e', 'd', 'B', 'o', 'o', 'l', 's', '\0',
  /*  4218 - "testOnly.onePingOneBool" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'o', 'n', 'e', 'P', 'i', 'n', 'g', 'O', 'n', 'e', 'B', 'o', 'o', 'l', '\0',
  /*  4242 - "testOnly.meaningOfLife" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'e', 'a', 'n', 'i', 'n', 'g', 'O', 'f', 'L', 'i', 'f', 'e', '\0',
  /*  4265 - "testOnlyIpc.aCounter" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'C', 'o', 'u', 'n', 't', 'e', 'r', '\0',
  /*  4286 - "testOnlyIpc.aBool" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'B', 'o', 'o', 'l', '\0',
  /*  4304 - "testOnlyIpc.aDate" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'D', 'a', 't', 'e', '\0',
  /*  4322 - "testOnlyIpc.aString" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'S', 't', 'r', 'i', 'n', 'g', '\0',
  /*  4342 - "testOnlyIpc.aMemoryDist" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'M', 'e', 'm', 'o', 'r', 'y', 'D', 'i', 's', 't', '\0',
  /*  4366 - "testOnlyIpc.aTimingDist" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'T', 'i', 'm', 'i', 'n', 'g', 'D', 'i', 's', 't', '\0',
  /*  4390 - "testOnlyIpc.aCustomDist" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'C', 'u', 's', 't', 'o', 'm', 'D', 'i', 's', 't', '\0',
  /*  4414 - "testOnlyIpc.aStringList" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'S', 't', 'r', 'i', 'n', 'g', 'L', 'i', 's', 't', '\0',
  /*  4438 - "testOnlyIpc.anEvent" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'n', 'E', 'v', 'e', 'n', 't', '\0',
  /*  4458 - "testOnlyIpc.eventWithExtra" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'e', 'v', 'e', 'n', 't', 'W', 'i', 't', 'h', 'E', 'x', 't', 'r', 'a', '\0',
  /*  4485 - "testOnlyIpc.noExtraEvent" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'n', 'o', 'E', 'x', 't', 'r', 'a', 'E', 'v', 'e', 'n', 't', '\0',
  /*  4510 - "testOnlyIpc.aUuid" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'U', 'u', 'i', 'd', '\0',
  /*  4528 - "testOnlyIpc.aLabeledCounter" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'L', 'a', 'b', 'e', 'l', 'e', 'd', 'C', 'o', 'u', 'n', 't', 'e', 'r', '\0',
  /*  4556 - "testOnlyIpc.anotherLabeledCounter" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'n', 'o', 't', 'h', 'e', 'r', 'L', 'a', 'b', 'e', 'l', 'e', 'd', 'C', 'o', 'u', 'n', 't', 'e', 'r', '\0',
  /*  4590 - "testOnlyIpc.aQuantity" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'Q', 'u', 'a', 'n', 't', 'i', 't', 'y', '\0',
  /*  4612 - "testOnlyIpc.irate" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'i', 'r', 'a', 't', 'e', '\0',
  /*  4630 - "testOnlyIpc.rateWithExternalDenominator" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'r', 'a', 't', 'e', 'W', 'i', 't', 'h', 'E', 'x', 't', 'e', 'r', 'n', 'a', 'l', 'D', 'e', 'n', 'o', 'm', 'i', 'n', 'a', 't', 'o', 'r', '\0',
  /*  4670 - "testOnlyIpc.anExternalDenominator" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'n', 'E', 'x', 't', 'e', 'r', 'n', 'a', 'l', 'D', 'e', 'n', 'o', 'm', 'i', 'n', 'a', 't', 'o', 'r', '\0',
  /*  4704 - "testOnlyIpc.aUrl" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'U', 'r', 'l', '\0',
  /*  4721 - "testOnlyJog.aCounter" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'J', 'o', 'g', '.', 'a', 'C', 'o', 'u', 'n', 't', 'e', 'r', '\0',
  /*  4742 - "testOnlyJog.anEvent" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'J', 'o', 'g', '.', 'a', 'n', 'E', 'v', 'e', 'n', 't', '\0',
};


static_assert(sizeof(gMetricStringTable) < 4294967296, "Metric string table is too large.");

const metric_entry_t sMetricByNameLookupEntries[] = {
  576461491037802956ull,
  576461486742835632ull,
  2305843447300360853ull,
  576460915512181870ull,
  8646912027580699118ull,
  5188146976889242954ull,
  4611686357729806492ull,
  2305843503134936087ull,
  6341068485791057237ull,
  6917529474317683401ull,
  2305843387170818372ull,
  576460885447410583ull,
  1152922260521095742ull,
  1729382312744845782ull,
  1729383008529551894ull,
  2882303856006398696ull,
  576460979936691643ull,
  2882303877481235265ull,
  576461044361201562ull,
  2305843395760753008ull,
  2305843726473236822ull,
  4611686533823466642ull,
  576461031476299563ull,
  576461040066234228ull,
  2305843400055720325ull,
  576461027181332234ull,
  4611686374909675755ull,
  6917529607461670488ull,
  1729382282680074399ull,
  576460971346756991ull,
  5764607703422862540ull,
  6917529517267356666ull,
  8646911812832333049ull,
  3458764651259495338ull,
  2305843730768204138ull,
  2305843374285916441ull,
  5188146959709373685ull,
  5188147011248981518ull,
  9223372569430723877ull,
  10952755041089360388ull,
  576460898332312567ull,
  2882303843121496748ull,
  576461396548521913ull,
  2882303886071169903ull,
  5188146985479177574ull,
  4611686190226080923ull,
  9799833334619049362ull,
  4611686692737257594ull,
  5188146964004341002ull,
  2305843477365132130ull,
  6341068571690403769ull,
  2305843404350687640ull,
  10376294198591623187ull,
  4611686602542943847ull,
  6917529508677422011ull,
  4035225927548932141ull,
  576461005706495569ull,
  2305843778012844678ull,
  6917529332583761930ull,
  2305843451595328171ull,
  3458764655554462672ull,
  1729382295564976418ull,
  5188147019838916152ull,
  6917529504382454683ull,
  8646911808537365719ull,
  4611686353434839179ull,
  576460949871920430ull,
  576461010001462900ull,
  1729382299859943763ull,
  4035225931843899457ull,
  1729382278385107057ull,
  4035225682735794701ull,
  9799833502122774846ull,
  6917529500087487358ull,
  2305843434415458885ull,
  576461254814600260ull,
  2305843357106047178ull,
  4611686379204643066ull,
  3458765162360606681ull,
  5188147183047674357ull,
  4035225871714356974ull,
  3458765183835443290ull,
  576461018591397569ull,
  2882303873186267952ull,
  5188147174457739708ull,
  2305843331336243284ull,
  576460902627279894ull,
  2882303868891300638ull,
  8070451039054072927ull,
  5764608223113908470ull,
  2305843082228138627ull,
  4611686344844904546ull,
  1729382304154911103ull,
  576460954166887740ull,
  6917529568806964598ull,
  6917529478612650726ull,
  6917529723425788130ull,
  1729382291270009076ull,
  2305843438710426207ull,
  6917529341173696558ull,
  5188147178752707034ull,
  576460988526626290ull,
  6917529336878729240ull,
  2882303881776202587ull,
  4611686589658041905ull,
  10376294194296655861ull,
  2882303946200712415ull,
  2305843408645654954ull,
  576461035771266894ull,
  2305843378580883755ull,
  2305843327041275974ull,
  2305843443005393529ull,
  8070451043349040248ull,
  6341068984007266598ull,
  2882303847416464059ull,
  4611686606837911173ull,
  1729382308449878440ull,
  576461022886364905ull,
  2305843391465785686ull,
  2305843352811079862ull,
  576461250519632947ull,
  1152921513196781593ull,
  4611686645492617048ull,
  7493990239506008867ull,
  4611686593953009225ull,
  7493990265275812824ull,
  8070451262392373662ull,
  2882303838826529425ull,
  4611686581068107271ull,
  1729382317039813124ull,
  576460881152443268ull,
  9799833338914016682ull,
  2305843163832517687ull,
  5188147015543948835ull,
  4611686538118433971ull,
  9223372655330070323ull,
  5188146946824471734ull,
  5188147535234994801ull,
  4035225433627690120ull,
  9799833403338526489ull,
  5188147453630615721ull,
  2305843735063171461ull,
  4611686705622159550ull,
  9223372573725691214ull,
  1729382286975041739ull,
  4611686572478172611ull,
  8646911963156189330ull,
  576460975641724317ull,
  2882303851711431378ull,
  6917529482907618054ull,
  576461014296430233ull,
  4611686585363074589ull,
  6917529491497552709ull,
  576461392253554586ull,
  2882304397172281220ull,
  1729382325629747807ull,
  4611686370614708441ull,
  5188147002659046873ull,
  2882303860301366008ull,
  2882304465891758350ull,
  4611686615427845828ull,
  2305843382875851064ull,
  1729382321334780469ull,
  4611686611132878502ull,
  2882303920430908492ull,
  4611686576773139940ull,
  8070451155018190662ull,
  7493990540153721440ull,
  5764608154394431341ull,
  1152921517491748908ull,
  9223372728344514768ull,
  1729382274090139730ull,
  2305843339926177911ull,
  576460756598390784ull,
  2305843309861406687ull,
  2882303864596333324ull,
  5188147372026236637ull,
  2305843430120491565ull,
  6917529637526441731ull
};



static Maybe<uint32_t>
MetricByNameLookup(const nsACString& aKey)
{
  static const uint16_t BASES[] = {
      15,   2,   5,   2,   2,  63,   2,  12,  30,   9,   2,   5,  15,   1,  17,  13,
       7,   6,   4,   3,  22,  23,   1,  59,  28,   2,  16,   9,   7,  43,   1,  34,
      23,  36,   1,  39,  46,  23,  86,   1, 208,  34,   5,   4,  18, 271,  46,  56,
       7,   1,  28,  85,   3,   6,  78,   7,  16,   1,   0, 497,   0,   0,  10,  77,
  };
  

  const char* bytes = aKey.BeginReading();
  size_t length = aKey.Length();
  auto& entry = mozilla::perfecthash::Lookup(bytes, length, BASES,
                                             sMetricByNameLookupEntries);
  return metric_result_check(aKey, entry);
}


/**
 * Get a category's name from the string table.
 */
static const char* GetCategoryName(category_entry_t entry) {
  MOZ_ASSERT(entry < sizeof(gCategoryStringTable), "Entry identifier offset larger than string table");
  return &gCategoryStringTable[entry];
}

/**
 * Get a metric's identifier from the string table.
 */
static const char* GetMetricIdentifier(metric_entry_t entry) {
  uint32_t offset = GLEAN_OFFSET(entry);
  MOZ_ASSERT(offset < sizeof(gMetricStringTable), "Entry identifier offset larger than string table");
  return &gMetricStringTable[offset];
}

/**
 * Check that the found entry is pointing to the right key
 * and return it.
 * Or return `Nothing()` if the entry was not found.
 */
static Maybe<uint32_t> category_result_check(const nsACString& aKey, category_entry_t entry) {
  if (MOZ_UNLIKELY(entry > sizeof(gCategoryStringTable))) {
    return Nothing();
  }
  if (aKey.EqualsASCII(gCategoryStringTable + entry)) {
    return Some(entry);
  }
  return Nothing();
}

/**
 * Check if the found entry index is pointing to the right key
 * and return the corresponding metric ID.
 * Or return `Nothing()` if the entry was not found.
 */
static Maybe<uint32_t> metric_result_check(const nsACString& aKey, uint64_t entry) {
  uint32_t metricId = entry >> GLEAN_INDEX_BITS;
  uint32_t offset = GLEAN_OFFSET(entry);

  if (offset > sizeof(gMetricStringTable)) {
    return Nothing();
  }

  if (aKey.EqualsASCII(gMetricStringTable + offset)) {
    return Some(metricId);
  }

  return Nothing();
}


#undef GLEAN_INDEX_BITS
#undef GLEAN_ID_BITS
#undef GLEAN_TYPE_ID
#undef GLEAN_METRIC_ID
#undef GLEAN_OFFSET

} // namespace mozilla::glean
#endif // mozilla_GleanJSMetricsLookup_h
